@@ -5,4 +5,7 @@ from .constants import *
 from .kernels import *
 from .landscape import *
 from .matrices import *
-from .trap import *
+from .plots import *
+from .optimization import *
+from .pointProcess import *
+from .network import *
\ No newline at end of file
