@@ -1,3 +1,7 @@
+'''Constants used across the pkg (colors, symbols, bio).
+
+'''
+
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
@@ -32,7 +36,13 @@ PINK_NAVY = col.colorPaletteFromHexList(['#e0c3fc',  '#00296b'])
 """Pink to Navy Blue cmap."""
 
 TRP_COLS = {
-    0: '#f7258515', 1: '#5ddeb115', 2: '#fe5f5515', 
+    0: '#f7258515', 1: '#5ddeb125', 2: '#fe5f5515', 
     3: '#f038ff15', 4: '#e2ef7015', 5: '#9381ff15', 
 }
-"""Base colors for trap types."""
\ No newline at end of file
+"""Base colors for trap types."""
+
+LAND_TUPLES = (
+    # ('110m', '#dfe7fdAA', 30), ('50m', '#dfe7fdAA', 30), 
+    ('10m', '#dfe7fd55', 30), ('10m', '#ffffffDD', 5)
+)
+"""Base colors for land boundaries."""
\ No newline at end of file
