@@ -1,3 +1,7 @@
+'''Main object with sites, traps, and masks to optimize and visualize the landscape to be analyzed.
+
+'''
+
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
@@ -13,6 +17,7 @@ import MGSurvE.kernels as krn
 import MGSurvE.plots as plt
 import MGSurvE.optimization as opt
 import MGSurvE.pointProcess as pts
+import MGSurvE.auxiliary as aux
 import warnings
 warnings.filterwarnings('ignore', 'The iteration is not making good progress')
 
@@ -20,9 +25,8 @@ class Landscape:
     """ Stores the information for a mosquito landscape. Works with different point-types in the form of matrices and coordinates.
     
     Parameters:
-        points (pandas dataframe): Sites coordinates in (x, y) or (lon, lat) format.
-        pointTypes (numpy array): Sites types in the same order and length as the points coordinates.
-               
+        points (pandas dataframe): Sites positions with mandatory {x,y} coordinates and optional {t: type, a: attractiveness} values for each site in the landscape.
+
         kernelFunction (function): Function that determines de relationship between distances and migration probabilities.
         kernelParams (dict): Parameters required for the kernel function to determine migration probabilities.
         maskingMatrix (numpy array): Matrix that determines the probability of shifting from one point-type to another one (squared with size equal to the number of point types). If None, every point-type transition is equiprobable.
@@ -33,19 +37,12 @@ class Landscape:
 
         distanceFunction (function): Function that takes two points in the landscape and calculates the distance between them.
 
-    Attributes:
-        pointsCoords (numpy array): 
-        pointNumber (int): Number of sites present in the environment.
-        geometryType (str): Type of geometry being analyzed (cartesian "xy" or lat-lon "ll")
+        traps (pandas dataframe): Traps positions with mandatory {x,y} coordinates and optional {t: trap type integer, f: fixed bool (immovable)}.
+        trapsKernels (dict): Traps' kernels functions and parameters in dictionary form (where the indices must match the "t" values in the traps dataframe).
+        trapsMask (numpy array): Traps' masking matrix to make traps act upon mosquitos searching for a specific resource (shape: {trapsNum, sitesNum}).
+        trapsRadii (list): List of probability values at which we want rings to be plotted in dataviz functions.
 
-        distanceMatrix (numpy array): Distances amongst the points in the landscape.
-        migrationMatrix (numpy array): Distance-based migration probabilities amongst the points in the landscape.
-        maskedMigrationMatrix (numpy array): Point-type based migration probabilities amongst the points in the landscape.
-
-    Methods:
-        calcPointsDistances: Calculates the distancesMatrix amongst the points (in place). Uses the distanceFunction to calculate the distanceMatrix internally.
-        calcPointsMigration: Calculates the migrationMatrix amongst the points (in place). Uses the kernelFunction and kernelParams to generate the migrationMatrix.
-        calcPointsMaskedMigration: Calculates the maskedMigrationMatrix depending on point-type (in place). Uses the maskingMatrix to bias the migrationMatrix and store the results in the maskedMigrationMatrix.
+        landLimits (tuple): Landscape's bounding box.
     """
     ###########################################################################
     # Initializers
@@ -53,7 +50,8 @@ class Landscape:
     def __init__(self, 
         points, 
         maskingMatrix=None,
-        
+        attractionVector=None,
+
         distanceMatrix=None, 
         distanceFunction=None, 
         
@@ -74,16 +72,15 @@ class Landscape:
         trapsMask=None,
         trapsRadii=[.25, .2, .1, .05],
 
-        repellents=None,
-        repellentsKernels={
-            0: {'kernel': krn.exponentialDecay, 'params': cst.BASIC_EXP_TRAP}
-        }
+        landLimits=None,
+        populations=None
     ):
         """Constructor method
         """
         self.kernelFunction = kernelFunction
         self.kernelParams = kernelParams
         self.maskingMatrix = maskingMatrix
+        self.attractionVector = attractionVector
         self.pointNumber = len(points)
         self.trapsCoords = None
         self.trapsTypes = None
@@ -95,6 +92,10 @@ class Landscape:
         self.trapsFixed= None
         self.fundamentalMatrix = None
         self.trapsMask = None
+        self.distanceFunction = distanceFunction
+        self.populations = populations
+        self.latlon = False
+        self.landLimits = landLimits
         # Check and define coordinates ----------------------------------------
         ptsHead = set(points.columns)
         if ('x' in ptsHead) and ('y' in ptsHead):
@@ -105,8 +106,9 @@ class Landscape:
         elif ('lat' in ptsHead) and ('lon' in ptsHead):
             self.geometryType = 'll'
             self.pointCoords = np.asarray(points[['lon', 'lat']])
+            self.latlon = True
             if distanceFunction is None:
-                self.distanceFunction = vin.vincenty
+                self.distanceFunction = aux.vincentyDistance
         else:
             raise Exception(
                 '''Check the landscape type! 
@@ -124,6 +126,12 @@ class Landscape:
             self.maskingMatrix = np.full((ptNum, ptNum), 1)
         else:
             self.maskingMatrix = np.asarray(maskingMatrix)
+        # If no attraction vector is provided, generate a dummy one -----------
+        if attractionVector is None:
+            ptNum = len(set(self.pointTypes))
+            self.attractionVector = np.full(ptNum, 1)
+        else:
+            self.attractionVector = np.asarray(attractionVector)
         # Init distance matrix ------------------------------------------------
         if distanceMatrix is None:
             self.calcPointsDistances()
@@ -138,7 +146,20 @@ class Landscape:
         if (maskedMigrationMatrix is None) and (self.pointTypes is not None):
             self.calcPointsMaskedMigration()
         else:
-            self.maskedMigration = np.asarray(maskedMigration)
+            self.maskedMigration = np.asarray(maskedMigrationMatrix)
+        # Setup population sizes ----------------------------------------------
+        if (self.populations is not None):
+            if (self.populations.shape[0] != self.pointNumber):
+                raise Exception(
+                    '''Number of populations ({}) is not the same as 
+                    number of sites ({}).
+                    '''.format(self.populations.shape[0], self.pointNumber)
+                )
+        # Set land limits -----------------------------------------------------
+        if self.landLimits is None:
+            self.landLimits = self.getBoundingBox()
+        else:
+            self.landLimits = landLimits
         # Init traps locations ------------------------------------------------
         if (traps is not None):
             tpsHead = set(traps.columns)
@@ -243,6 +264,11 @@ class Landscape:
             self.trapsMigration = mat.genVoidFullMigrationMatrix(
                 self.maskedMigration, self.trapsNumber
             )
+            mskShape = (
+                len(set(self.trapsTypes)), 
+                len(set(self.pointTypes))
+            )
+            self.trapsMask = np.full(mskShape, 1)
         self.calcTrapsDistances()
         self.calcTrapsMigration()
         self.updateTrapsRadii(self.trapsRadii)
@@ -251,7 +277,8 @@ class Landscape:
         for k in list(tker.keys()):
             tker[k].update({
                 'radii': [
-                    krn.nSolveKernel(tker[k], d, guess=0) for d in probValues
+                    krn.nSolveKernel(tker[k], d, guess=0, latlon=self.latlon) 
+                    for d in probValues
                 ]
             })
         self.trapsKernels = tker
@@ -299,7 +326,7 @@ class Landscape:
     def plotSites(self, 
             fig, ax, 
             markers=cst.MKRS, colors=cst.MCOL,
-            size=200, edgecolors='w', linewidths=2,
+            size=350, edgecolors='w', linewidths=2,
             zorder=5, **kwargs
         ):
         """Plots the sites coordinates.
@@ -307,7 +334,7 @@ class Landscape:
         (fig, ax) = plt.plotSites(
             fig, ax, 
             self.pointCoords, self.pointTypes,
-            colors=colors, size=size, 
+            colors=colors, size=size, markers=markers,
             edgecolors=edgecolors, linewidths=linewidths,
             zorder=zorder, **kwargs
         )
@@ -345,7 +372,7 @@ class Landscape:
         )
         return (fig, ax)
     def plotTraps(self,
-            fig, ax,
+            fig, ax, 
             colors=cst.TRP_COLS, marker="X",
             edgecolor='w', lws=(2, 0), ls=':',
             size=300, zorders=(25, -5),
@@ -354,7 +381,7 @@ class Landscape:
         """Plots the traps locations.
         """
         (fig, ax) = plt.plotTraps(
-            fig, ax,
+            fig, ax, 
             self.trapsCoords, self.trapsTypes, 
             self.trapsKernels, self.trapsFixed,
             colors=colors, marker=marker,
@@ -364,7 +391,7 @@ class Landscape:
         )
         return (fig, ax)
     def plotTrapsNetwork(self,
-            fig, ax,
+            fig, ax, 
             lineColor='#f72585', lineWidth=20, 
             alphaMin=.5, alphaAmplitude=1.5,
             zorder=0, **kwargs
@@ -372,7 +399,7 @@ class Landscape:
         """Plots the traps networks.
         """
         (fig, ax) = plt.plotTrapsNetwork(
-            fig, ax,
+            fig, ax, 
             self.trapsMigration, self.trapsCoords, self.pointCoords,
             lineColor=lineColor, lineWidth=lineWidth, 
             alphaMin=alphaMin, alphaAmplitude=alphaAmplitude,
@@ -385,12 +412,18 @@ class Landscape:
             edgecolors='black'
         ):
         (fig, ax)= plt.plotDirectedNetwork(
-            fig, ax,
+            fig, ax, 
             sites=self.pointCoords, pTypes=self.pointTypes,
             transMtx=self.migrationMatrix,
             markers=markers, colors=colors, edgecolors=edgecolors
         )
         return (fig, ax)
+    def plotLandBoundary(self,
+            fig, ax, 
+            landTuples=cst.LAND_TUPLES
+        ):
+        (fig, ax) = plt.plotLandBoundary(fig, ax, landTuples=landTuples)
+        return (fig, ax)
     ###########################################################################
     # Auxiliary Methods
     ###########################################################################
